#//-----------------------------------------------
#//TUSB-xx16ADSM-S2Z用Pythonサンプルプログラム
#// 連続取り込み処理
#//
#//2024_03_　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------

import USBADSMS

import tkinter as tk
import tkinter.ttk as ttk
from tkinter import messagebox
from tkinter import filedialog
import threading

DevId = 0
OpenState = False
dfile = None            #保存ファイル
TimerStopReq = False    #タイマ停止要求フラグ
SampledLen = 0          #取り込み済みデータ長
ReqDataLen = 0          #最大取り込み設定値(0の時無制限)
ChLen = 0               #取り込みチャンネル数
SamplingBuffer = list() #取り込みデータ保存リスト

#/// Sub Functions //////////////////////////////

# Widgetの表示を設定する
# stat -> 'Close' , 'OpenStop' , 'OpenRun'
def WidgetState(stat):
    EnClose = tk.NORMAL if( stat == 'Close' ) else tk.DISABLED
    EnOpenStop = tk.NORMAL if( stat == 'OpenStop' ) else tk.DISABLED
    EnOpenRun = tk.NORMAL if( stat == 'OpenRun' ) else tk.DISABLED
    OpenBtn['state'] = EnClose
    id_combo['state'] = EnClose
    CloseBtn['state'] = EnOpenStop
    StartBtn['state'] = EnOpenStop
    for chk in ChChk: chk['state'] = EnOpenStop
    for cmb in range_combo: cmb['state'] = EnOpenStop
    trgsel_combo['state'] = EnOpenStop
    trglev_entry['state'] = EnOpenStop
    trgch_combo['state'] = EnOpenStop
    clkint_rbtn['state'] = EnOpenStop
    clkext_rbtn['state'] = EnOpenStop
    clkintv_entry['state'] = EnOpenStop
    prelen_entry['state'] = EnOpenStop
    savelen_entry['state'] = EnOpenStop
    TrgBtn['state'] = EnOpenRun
    StopBtn['state'] = EnOpenRun

    if stat != 'Close' :
        form.protocol('WM_DELETE_WINDOW',lambda: 'pass' )
    else :
        form.protocol('WM_DELETE_WINDOW',lambda: form.destroy())

# 文字列strを表示する
def ShowInfo(str):
    messagebox.showinfo("",str)

# ステータス表示をする
def ShowStat(str):
    StatLbl['text'] = str

# ステータス表示に文字列を追加する
def AddStat(str):
    StatLbl['text'] += str

# リターンコードretcodeに対応する文字列を表示する
def ShowRet(retcode):
    ShowInfo(USBADSMS.GetErrMessage(retcode))

# datの指定ビットbitが1(True)か0(False)を判定する
def IsBitSet(dat,bit):
    if( ( ( dat >> bit ) & 0b1 ) == 0b1 ): return True
    else: return False

# AD変換データから電圧値へ変換する
#オフセットテーブル
AdOffset = {
    "±10V":-10.0,
    "±5V":-5.0,
    "±2.5V":-2.5,
    "±1V":-1.0,
    "±0.5V":-0.5,
    "±0.25V":-0.25,
    "±0.1V":-0.1,
    "10V":0.0,
    "5V":0.0,
    "2.5V":0.0,
    "1V":0.0,
    "0.5V":0.0,
    "0.25V":0.0,
    "0.1V":0.0
}
#スパンテーブル
AdSpan = {
    "±10V":20.0,
    "±5V":10.0,
    "±2.5V":5.0,
    "±1V":2.0,
    "±0.5V":1.0,
    "±0.25V":0.5,
    "±0.1V":0.2,
    "10V":10.0,
    "5V":5.0,
    "2.5V":2.5,
    "1V":1.0,
    "0.5V":0.5,
    "0.25V":0.25,
    "0.1V":0.1
}

#取り込みチャンネルの選択とレンジ選択の取得
def GetChSets():
    chpattern = list()
    chrange = list()
    for ch in range(16):
        if ValChEn[ch].get():
            chpattern.append(ch)
            chrange.append(range_combo[ch].current())
    return len(chpattern),chpattern,chrange

#トリガレベルの出力
def GetTrgLevel():
    if( not trglev_entry.get().isdecimal() ): return False
    ret = int( trglev_entry.get())
    if ret < 1 or ret > 4094 : return False
    return ret

#サンプリングクロック設定の取得
def GetSmplClk():
    if( not clkintv_entry.get().isdecimal() ): return False
    ret = int( clkintv_entry.get() )
    if ret < 10 or ret > 16777215 : return False
    return ret

#プレトリガ長設定の取得
def GetPreTrgLen():
    if( not prelen_entry.get().isdecimal() ): return False
    ret = int( prelen_entry.get() )
    if ret < 0 or ret > 10000000 : return False
    return ret

#取り込み要求データ長の取得
def GetReqSaveLen():
    if( not savelen_entry.get().isdecimal() ): return False
    ret = int( savelen_entry.get() )
    if ret < 0 or ret > 100000000: return False
    return ret

#/// interval timer ///////////////////////////
# 取得したデータを一時保存する
def DataCapture( Data , Leng ):
    global SamplingBuffer
    for ch in range(ChLen):
        for cnt in range(Leng):
            SamplingBuffer[ch].append(Data[ch*Leng+cnt])
    return

# 一時データをファイルに書き込んでファイルを閉じる
def WriteDataAndClose():
    for cnt in range(SampledLen):
        for ch in range(ChLen):
            dfile.write(str(SamplingBuffer[ch][cnt]))
            if ch == ( ChLen - 1 ):
                dfile.write("\n")
            else:
                dfile.write(",")
    dfile.close()
    AddStat("\nサンプリング終了\n" + "データ数" + str(SampledLen))
    WidgetState('OpenStop')
    ret = USBADSMS.Tusbadsms_Sampling_Stop(DevId)
    if( ret != 0 ):ShowRet(ret)
    return

# 取り込み状態を確認して処理を行う
def CheckSample():
    global SampledLen
    ret,Status,Ovf,Leng = USBADSMS.Tusbadsms_Status_Read(DevId)
    if( ret != 0 ): ShowRet(ret)
    if Status == 0:#停止中
        ShowStat("停止中です")
    elif Status == 1:#トリガ待ち
        ShowStat("トリガ待ちです")
    elif Status == 2:#サンプリング中
        if Ovf != 0 :#オーバーフローの時
            ShowStat("バッファオーバーフローです")
            WriteDataAndClose()
            return False
        if ReqDataLen != 0 and (SampledLen + Leng) > ReqDataLen :
            Leng = ReqDataLen - SampledLen
            ret,AdData,RLeng = USBADSMS.Tusbadsms_Memory_Read(DevId,Leng)
            DataCapture(AdData,RLeng)
            SampledLen += RLeng
            WriteDataAndClose()
            return False
        else:
            ret,AdData,RLeng = USBADSMS.Tusbadsms_Memory_Read(DevId,Leng)
            DataCapture(AdData,RLeng)
            SampledLen += RLeng
            ShowStat("サンプリング中です\n"+"データ数"+str(SampledLen))
    return True

# 定周期タイマー処理
def intervalTimer():
    if(TimerStopReq):
        WriteDataAndClose()
    else:
        if CheckSample():
            threading.Timer(0.1,intervalTimer).start()

#/// Button event handlers /////////////////////////
#デバイスオープン
def Open_on():
    global DevId
    DevId = id_combo.current()
    ret = USBADSMS.Tusbadsms_Device_Open(DevId)
    if(ret == 0):
        WidgetState('OpenStop')
    else:
        ShowRet(ret)

#デバイスクローズ
def Close_on():
    USBADSMS.Tusbadsms_Device_Close(DevId)
    WidgetState('Close')

# 取り込み開始処理
def Start_on():
    sets = USBADSMS.adsms_setting()
    global ChLen
    ChLen,sets.ChPattern,sets.ChPatternRange = GetChSets()
    if ChLen == 0 : return
    sets.ChLen = ChLen
    sets.TriggerType = trgsel_combo.current()+1
    sets.TriggerLevel = GetTrgLevel()
    if type(sets.TriggerLevel) != int:
        ShowInfo("トリガレベル入力値が不正です")
        return
    sets.TriggerCh = trgch_combo.current()
    sets.ClockSel = ValClkSel.get()
    sets.SamplingClock = GetSmplClk()
    if type(sets.SamplingClock) != int:
        ShowInfo("クロック間隔設定値が不正です")
        return
    sets.PreTriggerLen = GetPreTrgLen()
    if type(sets.PreTriggerLen) != int:
        ShowInfo("プレトリガ設定値が不正です")
        return
    global ReqDataLen
    ReqDataLen = GetReqSaveLen()
    if type(ReqDataLen) != int:
        ShowInfo("保存数の設定値が不正です")
        return
    sets.TotalLen = 1000000
    
    filename = filedialog.asksaveasfilename(
        title = "保存ファイルを指定してください",
        filetypes = [("データファイル","*.csv")],
        defaultextension = "csv"
        )
    if len(filename) == 0 : return
    global dfile
    try:
        dfile = open(filename,"w")
    except OSError as e:
        ShowInfo(e)
    else:
        global SamplingBuffer
        SamplingBuffer = [[]for ch in range(ChLen)]
        ret = USBADSMS.Tusbadsms_Sample_Start(DevId,sets)
        if ret == 0 :
            global TimerStopReq
            TimerStopReq = False
            global SampledLen
            SampledLen = 0
            WidgetState('OpenRun')
            intervalTimer()
        else:
            ShowRet(ret)
            dfile.close()

#取り込みて停止
def Stop_on():
    TimerStopReq = True
    return

#トリガ発行
def Trigger_on():
    ret = USBADSMS.Tusbadsms_Sampling_Trigger(DevId)
    if ret != 0:
        ShowRet(ret)
    return

#/// form generator ///////////////////////////
form = tk.Tk()
form.geometry('520x492')
form.title('TUSB-xx16ADSM-S2Z Long Sample')

# buttons

OpenBtn = tk.Button(form, text = 'Open', command=Open_on)
OpenBtn.place(relwidth =0.187,relheight=0.06,relx=0.3,rely=0.9)

CloseBtn = tk.Button(form, text = 'Close', command=Close_on)
CloseBtn.place(relwidth =0.187,relheight=0.06,relx=0.53,rely=0.9)

StartBtn = tk.Button(form, text = 'Start', command=Start_on)
StartBtn.place(relwidth =0.25,relheight=0.06,relx=0.35,rely=0.67)

StopBtn = tk.Button(form, text = 'Stop', command=Stop_on)
StopBtn.place(relwidth =0.25,relheight=0.06,relx=0.65,rely=0.67)

TrgBtn = tk.Button(form, text = 'TRG', command=Trigger_on)
TrgBtn.place(relwidth =0.07,relheight=0.04,relx=0.9,rely=0.02)

#labels

IDLbl = tk.Label(form,text="ID")
IDLbl.place(relx=0.06,rely = 0.92)

ChLbl = tk.Label(form,text="Ch")
ChLbl.place(relx=0.038,rely = 0.02)

RangeLbl = tk.Label(form,text="Range")
RangeLbl.place(relx=0.14,rely = 0.02)

RangeLbl = tk.Label(form,text="Trigger Select")
RangeLbl.place(relx=0.32,rely = 0.02)
RangeLbl = tk.Label(form,text="Trigger Level(1-4094)")
RangeLbl.place(relx=0.32,rely = 0.11)
RangeLbl = tk.Label(form,text="Trigger Ch.")
RangeLbl.place(relx=0.32,rely = 0.20)
RangeLbl = tk.Label(form,text="Clock Select")
RangeLbl.place(relx=0.32,rely = 0.29)
RangeLbl = tk.Label(form,text="Clock Interval")
RangeLbl.place(relx=0.32,rely = 0.38)
RangeLbl = tk.Label(form,text="Pre Trigger Leng.")
RangeLbl.place(relx=0.32,rely = 0.47)
RangeLbl = tk.Label(form,text="Save Leng Limit")
RangeLbl.place(relx=0.32,rely = 0.56)

StatLbl = tk.Label(form,text="",borderwidth=1,relief="ridge")
StatLbl.place(relwidth = 0.6,relheight = 0.1,relx=0.32,rely = 0.76)

# list boxes

id_option = list(range(16))
id_combo = ttk.Combobox (form, values = id_option,state="readonly")
id_combo.place(relwidth= 0.11,relx = 0.12,rely=0.92)
id_combo.current(0)

range_option = ["±10V","±5V","±2.5V","±1V","±0.5V","±0.25V","±0.1V",\
                "10V","5V","2.5V","1V","0.5V","0.25V","0.1V"]
range_combo = list()
for ch in range(16):
    range_combo.append(ttk.Combobox(form,values = range_option,state="readonly"))
    range_combo[ch].place(relwidth=0.12,relx=0.14,rely=0.07 + 0.05*ch)
    range_combo[ch].current(0)

trgsel_option = ["Soft","Ext. Digital(Up)","Ext. Digital(Down)","Edge(Up)","Edge(Down)","Level(Up)","Level(Down)"]
trgsel_combo = ttk.Combobox(form,values= trgsel_option,state="readonly")
trgsel_combo.place(relwidth=0.27, relx=0.6,rely=0.02)
trgsel_combo.current(0)

trgch_option = list(range(16))
trgch_combo = ttk.Combobox (form, values = id_option,state="readonly")
trgch_combo.place(relwidth= 0.15,relx = 0.6,rely=0.20)
trgch_combo.current(0)

# Entrys
trglev_entry = tk.Entry(form)
trglev_entry.place(relwidth=0.2,relx =0.6,rely= 0.11)
trglev_entry.insert(tk.END,"2048")

clkintv_entry = tk.Entry(form)
clkintv_entry.place(relwidth=0.3,relx =0.6,rely= 0.38)
clkintv_entry.insert(tk.END,"10")

prelen_entry = tk.Entry(form)
prelen_entry.place(relwidth=0.3,relx =0.6,rely= 0.47)
prelen_entry.insert(tk.END,"0")

savelen_entry = tk.Entry(form)
savelen_entry.place(relwidth=0.3,relx =0.6,rely= 0.56)
savelen_entry.insert(tk.END,"1000")

# check boxes

ValChEn = list()
ChChk = list()
for ch in range(16):
    ValChEn.append(tk.BooleanVar())
    ValChEn[ch].set(False)
    ChChk.append(tk.Checkbutton(form,text=str(ch),variable=ValChEn[ch]))
    ChChk[ch].place(relx=0.038,rely=0.07 + 0.05*ch)

# radio buttons
ValClkSel = tk.IntVar()
ValClkSel.set(0)

clkint_rbtn = tk.Radiobutton(form,text="Internal",value=0,variable=ValClkSel)
clkint_rbtn.place(relx = 0.6,rely = 0.29)

clkext_rbtn = tk.Radiobutton(form,text="External",value=1,variable=ValClkSel)
clkext_rbtn.place(relx = 0.75,rely = 0.29)

#/////////////////////////////////////////////////

#//// Start

WidgetState('Close')
form.mainloop()     # フォーム画面表示
if(OpenState):
    USBADSMS.Tusbadsms_Device_Close(DevId)
