#//-----------------------------------------------
#//TUSB-xx16ADSM-S2Z用Pythonサンプルプログラム
#// SigleSample機能使用
#//
#//2024_03_　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------

import USBADSMS

import tkinter as tk
import tkinter.ttk as ttk
from tkinter import messagebox

DevId = 0           #デバイスID 0~15
OpenState = False   #デバイスのオープン状態

# color define 入力値表示用
COL_RED = "#FFF000000"      #赤
COL_DRED = "#800000000"     #暗い赤

#/// Sub Functions //////////////////////////////

# Widgetの表示を設定する
# stat -> 'Close' , 'Open'
def WidgetState(stat):
    EnClose = tk.NORMAL if( stat == 'Close' ) else tk.DISABLED
    EnOPen = tk.NORMAL if( stat == 'Open' ) else tk.DISABLED
    OpenBtn['state'] = EnClose
    id_combo['state'] = EnClose
    CloseBtn['state'] = EnOPen
    DigOutBtn['state'] = EnOPen
    DigInBtn['state'] = EnOPen
    SingSmplBtn['state'] = EnOPen
    range_combo['state'] = EnOPen
    ch_combo['state'] = EnOPen
    DoChk0['state'] = EnOPen
    DoChk1['state'] = EnOPen

    if stat != 'Close' :
        form.protocol('WM_DELETE_WINDOW',lambda: 'pass' )
    else :
        form.protocol('WM_DELETE_WINDOW',lambda: form.destroy())

# 文字列strを表示する
def ShowInfo(str):
    messagebox.showinfo("",str)

# リターンコードretcodeに対応する文字列を表示する
def ShowRet(retcode):
    ShowInfo(USBADSMS.GetErrMessage(retcode))

# datの指定ビットbitが1(True)か0(False)を判定する
def IsBitSet(dat,bit):
    if( ( ( dat >> bit ) & 0b1 ) == 0b1 ): return True
    else: return False

# AD変換データから電圧値へ変換する
#オフセットテーブル
AdOffset = {
    "±10V":-10.0,
    "±5V":-5.0,
    "±2.5V":-2.5,
    "±1V":-1.0,
    "±0.5V":-0.5,
    "±0.25V":-0.25,
    "±0.1V":-0.1,
    "10V":0.0,
    "5V":0.0,
    "2.5V":0.0,
    "1V":0.0,
    "0.5V":0.0,
    "0.25V":0.0,
    "0.1V":0.0
}
#スパンテーブル
AdSpan = {
    "±10V":20.0,
    "±5V":10.0,
    "±2.5V":5.0,
    "±1V":2.0,
    "±0.5V":1.0,
    "±0.25V":0.5,
    "±0.1V":0.2,
    "10V":10.0,
    "5V":5.0,
    "2.5V":2.5,
    "1V":1.0,
    "0.5V":0.5,
    "0.25V":0.25,
    "0.1V":0.1
}
def CnvtToVolt(addata):
    range = range_combo.get()
    return addata * AdSpan[range] / 4096.0 + AdOffset[range]

#/// Button event handlers /////////////////////////
#デバイスオープン
def Open_on():
    global DevId
    DevId = id_combo.current()
    ret = USBADSMS.Tusbadsms_Device_Open(DevId)
    if(ret == 0):
        WidgetState('Open')
    else:
        ShowRet(ret)

#デバイスクローズ
def Close_on():
    USBADSMS.Tusbadsms_Device_Close(DevId)
    WidgetState('Close')

#デジタル出力
def DigOut_on():
    dob = 0b01 if(ValDo0.get()) else 0b00
    dob |= 0b10 if(ValDo1.get()) else 0b00
    ret = USBADSMS.Tusbadsms_Pio_Write(DevId,dob)
    if( ret):
        ShowRet(ret)

#デジタル入力
def DigIn_on():
    ret,dib = USBADSMS.Tusbadsms_Pio_Read(DevId)
    if( ret == 0 ):
        DiLed0['bg'] = COL_RED if(IsBitSet(dib,0)) else COL_DRED
        DiLed1['bg'] = COL_RED if(IsBitSet(dib,1)) else COL_DRED
    else:
        ShowRet(ret)

#AD変換データ取得
def SingSmpl_on():
    ch = ch_combo.current()
    range = range_combo.current()
    ret,addat = USBADSMS.Tusbadsms_Single_Sample(DevId,ch,range)
    if( ret == 0 ):
        VoltLbl["text"] = format(CnvtToVolt(addat),".4f")
        RawDLbl["text"] = format(addat)
    else:
        ShowRet(ret)

#/// form generator ///////////////////////////
form = tk.Tk()
form.geometry('570x182')
form.title('TUSB-xx16ADSM-S2Z Simple Sample')

# buttons

DigOutBtn = tk.Button(form, text = 'Digital Output', command=DigOut_on)
DigOutBtn.place(relwidth =0.15,relheight=0.14,relx=0.13,rely=0.18,anchor=tk.CENTER)

DigInBtn = tk.Button(form, text = 'Digital Input', command=DigIn_on)
DigInBtn.place(relwidth =0.15,relheight=0.14,relx=0.13,rely=0.4,anchor=tk.CENTER)

OpenBtn = tk.Button(form, text = 'Open', command=Open_on)
OpenBtn.place(relwidth =0.14,relheight=0.14,relx=0.37,rely=0.79,anchor=tk.CENTER)

CloseBtn = tk.Button(form, text = 'Close', command=Close_on)
CloseBtn.place(relwidth =0.14,relheight=0.14,relx=0.54,rely=0.79,anchor=tk.CENTER)

SingSmplBtn = tk.Button(form, text = 'Single Sample (A/D)', command=SingSmpl_on)
SingSmplBtn.place(relwidth =0.21,relheight=0.14,relx=0.59,rely=0.40,anchor=tk.CENTER)

#labels

IDLbl = tk.Label(form,text="ID")
IDLbl.place(relx=0.08,rely = 0.79,anchor=tk.CENTER)

RangeLbl = tk.Label(form,text="Range")
RangeLbl.place(relx=0.50,rely = 0.18,anchor=tk.CENTER)

ChLbl = tk.Label(form,text="Channel")
ChLbl.place(relx=0.77,rely = 0.18,anchor=tk.CENTER)

VLbl = tk.Label(form,text="V")
VLbl.place(relx=0.9,rely = 0.40,anchor=tk.CENTER)

VoltLbl = tk.Label(form,text="",borderwidth=1,relief="ridge")
VoltLbl.place(relwidth = 0.15,relheight = 0.14,relx=0.80,rely = 0.40,anchor=tk.CENTER)


RawDLbl = tk.Label(form,text="",borderwidth=1,relief="ridge")
RawDLbl.place(relwidth = 0.15,relheight = 0.14,relx=0.80,rely = 0.57,anchor=tk.CENTER)

# list boxes

id_option = list(range(16))
id_combo = ttk.Combobox (form, values = id_option,state="readonly")
id_combo.place(relwidth= 0.1,relheight =0.14,relx = 0.18,rely=0.79,anchor=tk.CENTER)
id_combo.current(0)

range_option = ["±10V","±5V","±2.5V","±1V","±0.5V","±0.25V","±0.1V",\
                "10V","5V","2.5V","1V","0.5V","0.25V","0.1V"]
range_combo = ttk.Combobox(form,values = range_option,state="readonly")
range_combo.place(relwidth=0.15,relheight=0.14,relx=0.62,rely=0.18,anchor=tk.CENTER)
range_combo.current(0)

ch_option = list(range(16))
ch_combo = ttk.Combobox(form,values = ch_option,state="readonly")
ch_combo.place(relwidth=0.1,relheight=0.14,relx=0.9,rely=0.18,anchor=tk.CENTER)
ch_combo.current(0)

# check boxes

ValDo0 = tk.BooleanVar()
ValDo0.set(False)
DoChk0 = tk.Checkbutton(form,text="0",variable =ValDo0)
DoChk0.place(relx=0.27,rely=0.18,anchor=tk.CENTER)

ValDo1 = tk.BooleanVar()
ValDo1.set(False)
DoChk1 = tk.Checkbutton(form,text="1",variable =ValDo1)
DoChk1.place(relx=0.37,rely=0.18,anchor=tk.CENTER)

# Shapes

DiLed0 = tk.Canvas(form,width = 15, height = 15, bg = "red")
DiLed0.place(relx = 0.27,rely = 0.4,anchor = tk.CENTER)

DiLed1 = tk.Canvas(form,width = 15, height = 15, bg = "red")
DiLed1.place(relx = 0.37,rely = 0.4,anchor = tk.CENTER)

#/////////////////////////////////////////////////

#//// Start

WidgetState('Close')
form.mainloop()     # フォーム画面表示
if(OpenState):
    USBADSMS.Tusbadsms_Device_Close(DevId)
