#//-----------------------------------------------
#//TUSB-xx12ADSM-S2Z用ドライバアクセス用API
#//Python 用モジュール
#//
#//2024_03_19　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------

from ctypes import *

#定義類
c_int32_max = 2 ** 31 - 1

#デバイス用DLLのロード  
tusbadsmsdll = cdll.LoadLibrary("TUSADSMS.DLL")

#グローバル変数
Set_Ch_Len = 0      # 取り込みチャンネル数

#デバイスドライバ関数の型定義  (これは直接呼ばない)
class _adsms_setting(Structure):
    _pack_ = 1
    _fields_ = [
        ('ChLen',c_ubyte),
        ('ChPattern',(c_ubyte*16)),
        ('ChPatternRange',(c_ubyte*16)),
        ('TriggerType',c_ubyte),
        ('TriggerLevel',c_short),
        ('TriggerCh',c_ubyte),
        ('ClockSel',c_ubyte),
        ('Padding',c_short),
        ('SamplingClock',c_int32),
        ('PreTriggerLen',c_int32),
        ('TotalLen',c_int32)
    ]
tusbadsmsdll.Tusbadsms_Device_Open.restype = c_short
tusbadsmsdll.Tusbadsms_Device_Open.argtypes = (c_short,)
tusbadsmsdll.Tusbadsms_Device_Close.restype = None
tusbadsmsdll.Tusbadsms_Device_Close.argtypes = (c_short,)
tusbadsmsdll.Tusbadsms_Pio_Write.restype = c_short
tusbadsmsdll.Tusbadsms_Pio_Write.argtypes = (c_short,c_ubyte)
tusbadsmsdll.Tusbadsms_Pio_Read.restype = c_short
tusbadsmsdll.Tusbadsms_Pio_Read.argtypes = (c_short,POINTER(c_ubyte))
tusbadsmsdll.Tusbadsms_Single_Sample.restype = c_short
tusbadsmsdll.Tusbadsms_Single_Sample.argtypes = (c_short,c_ubyte,c_ubyte,POINTER(c_short))
tusbadsmsdll.Tusbadsms_Status_Read.restype = c_short
tusbadsmsdll.Tusbadsms_Status_Read.argtypes = (c_short,POINTER(c_ubyte),POINTER(c_ubyte),POINTER(c_int32))
tusbadsmsdll.Tusbadsms_Memory_Read.restype = c_short
tusbadsmsdll.Tusbadsms_Memory_Read.argtypes = (c_short,POINTER(c_short),POINTER(c_int32))
tusbadsmsdll.Tusbadsms_Sampling_Stop.restype = c_short
tusbadsmsdll.Tusbadsms_Sampling_Stop.argtypes = (c_short,)
tusbadsmsdll.Tusbadsms_Memory_Clear.restype = c_short
tusbadsmsdll.Tusbadsms_Memory_Clear.argtypes = (c_short,)
tusbadsmsdll.Tusbadsms_Sampling_Trigger.restype = c_short
tusbadsmsdll.Tusbadsms_Sampling_Trigger.argtypes = (c_short,)
tusbadsmsdll.Tusbadsms_Sample_Start.restype = c_short
tusbadsmsdll.Tusbadsms_Sample_Start.argtypes = (c_short,POINTER(_adsms_setting))

#######  補助関数 (これも呼ばない)
def isint(n):
    return type(n) is int

def IsInRange(Val,Min,Max):
    if(isint(Val) == False ):
        return False
    elif(Val < Min):
        return False
    elif(Val > Max):
        return False
    else:
        return True

def IsValidId(DevId):
    return IsInRange(DevId,0,15)

######　機能関数の定義 
######  以下の関数を呼び出してデバイスを操作する
#DevIdは装置の設定ID

#設定クラス(DLL関数の設定用構造体として渡す)
class adsms_setting:
    def __init__(self
    ):
        self.ChLen = 1                   #チャンネル長 1~16
        self.ChPattern = list(range(1))  #チャンネルパターンリスト ChLen以上の長さ
        self.ChPatternRange = [0] * 1    #レンジリスト ChLen以上の長さ 
        self.TriggerType = 1             #トリガ種類　1:ソフト　2:外部ディジタル立上り 3:外部ディジタル立下り 4:立上エッジ 5:立下エッジ　6:上限レベル　7:下限レベル
        self.TriggerLevel = 0            #エッジやレベルの閾値
        self.TriggerCh = 0               #エッジやレベルを検出するチャンネル位置(パターンリストの要素番号)
        self.ClockSel = 0                #0:内部クロック　1:外部クロック
        self.SamplingClock = 10          #内部クロック間隔 10～16,777,215[uS]
        self.PreTriggerLen = 0           #プレトリガ長 0～10,000,000
        self.TotalLen = 1000             #全取込データバッファ 1～10,000,000
    

# デバイスオープン
def Tusbadsms_Device_Open(DevId):
    if( IsValidId(DevId) ):
        return tusbadsmsdll.Tusbadsms_Device_Open(DevId)
    else:
        return 8    #Invalid parameter
    
#デバイスクローズ
def Tusbadsms_Device_Close(DevId):
    if( IsValidId(DevId) ):
        tusbadsmsdll.Tusbadsms_Device_Close(DevId)
    else:
        return 8    #Invalid parameter

#Digital Out
def Tusbadsms_Pio_Write(DevId,WriteData):
    if( IsValidId(DevId) ):
        if(IsInRange(WriteData,0,3)):
            return tusbadsmsdll.Tusbadsms_Pio_Write(DevId,WriteData)
    return 8    #Invalid parameter

#Digital In
# 戻り値タプル (結果コード、読み込みデータ)
def Tusbadsms_Pio_Read(DevId):
    tmp = c_ubyte()
    if( IsValidId(DevId)):
        ret = tusbadsmsdll.Tusbadsms_Pio_Read(DevId,byref(tmp))
        if( ret != 0 ):
            return ret,0
        else:
            return 0,tmp.value
    else:
        return 8,0    #Invalid parameter

#AD 1回データ変換
#　戻り値タプル(結果コード、読み込みデータ)
def Tusbadsms_Single_Sample(DevId,Ch,Range):
    tmp = c_short()
    if( IsValidId(DevId)):
        if( IsInRange(Ch,0,15) and IsInRange(Range,0,13)):
            ret = tusbadsmsdll.Tusbadsms_Single_Sample(DevId,Ch,Range,byref(tmp))
            if( ret != 0 ):
                return ret,0
            else:
                return 0,tmp.value
    return 8,0    #Invalid parameter

# AD 連続取り込みステータス確認
# 戻り値タプル(結果コード、ステータス、オーバーフロービットパターン、データ長)
def Tusbadsms_Status_Read(DevId):
    tmpStatus = c_ubyte()
    tmpOvf = c_ubyte()
    tmpLeng = c_int32()
    if( IsValidId(DevId)):
        ret = tusbadsmsdll.Tusbadsms_Status_Read(DevId,byref(tmpStatus),byref(tmpOvf),byref(tmpLeng))
        if( ret != 0 ):
            return ret,0,0,0
        else:
            return 0,tmpStatus.value,tmpOvf.value,tmpLeng.value
    return 8,0,0,0    #Invalid parameter

# AD データ取り込み
# 引数 ReqLeng : 要求データ長
# 戻り値タプル (結果コード、データリスト、取得できたデータ長)
def Tusbadsms_Memory_Read(DevId,ReqLeng):
    if( IsValidId(DevId)):
        if( IsInRange(ReqLeng,0,c_int32_max)):
            tmpLen = c_int32(ReqLeng)
            tmpDataType = c_short * ( ReqLeng * Set_Ch_Len )
            tmpDataInst = tmpDataType()
            p_tmpData = cast(pointer(tmpDataInst),POINTER(c_short))
 #           p_tmpData = pointer(tmpDataInst)
            ret = tusbadsmsdll.Tusbadsms_Memory_Read(DevId,p_tmpData,byref(tmpLen))
            if( ret != 0 ):
                return ret,0,0
            else:
                ResultLeng = tmpLen.value
                Data = list()
                for count in range(ResultLeng * Set_Ch_Len):
                    Data.append(tmpDataInst[count])
                return 0,Data,ResultLeng
    return 8,0,0    #Invalid parameter

# AD 連続取り込み停止
def Tusbadsms_Sampling_Stop(DevId):
    if( IsValidId(DevId)):
        return tusbadsmsdll.Tusbadsms_Sampling_Stop(DevId)
    return 8    #Invalid parameter

# AD 連続取り込みメモリクリア
def Tusbadsms_Memory_Clear(DevId):
    if( IsValidId(DevId)):
        return tusbadsmsdll.Tusbadsms_Memory_Clear(DevId)
    return 8    #Invalid parameter

# AD 連続取り込み開始トリガ発行
def Tusbadsms_Sampling_Trigger(DevId):
    if( IsValidId(DevId)):
        return tusbadsmsdll.Tusbadsms_Sampling_Trigger(DevId)
    return 8    #Invalid parameter

# AD 連続取り込み開始
# 引数　SettingS 設定Structureクラス
def Tusbadsms_Sample_Start(DevId,SettingS : adsms_setting):
    global Set_Ch_Len
    if( IsValidId(DevId)):
        if( not IsInRange(SettingS.ChLen,1,16) ): return 8
        if( not IsInRange(len(SettingS.ChPattern),1,SettingS.ChLen)): return 8
        if( not IsInRange(len(SettingS.ChPatternRange),1,SettingS.ChLen)): return 8
        sets = _adsms_setting()
        sets.ChLen = SettingS.ChLen
        Set_Ch_Len = sets.ChLen
        for count in range(sets.ChLen): 
            sets.ChPattern[count] = SettingS.ChPattern[count]
            sets.ChPatternRange[count] = SettingS.ChPatternRange[count]
        sets.TriggerType = SettingS.TriggerType
        sets.TriggerLevel = SettingS.TriggerLevel
        sets.TriggerCh = SettingS.TriggerCh
        sets.ClockSel = SettingS.ClockSel
        sets.SamplingClock = SettingS.SamplingClock
        sets.PreTriggerLen = SettingS.PreTriggerLen
        sets.TotalLen = SettingS.TotalLen
        return tusbadsmsdll.Tusbadsms_Sample_Start(DevId,byref(sets))
    else:
        return 8    #Invalid parameter

#戻り値用エラーメッセージ
ErrStrDic = {
            0:"正常終了しました",
            1:"ID番号が不正です",
            2:"ドライバがインストールされていません",
            3:"すでにデバイスはオープンされています",
            4:"接続されている台数が多すぎます",
            5:"オープンできませんでした",
            6:"デバイスがみつかりません",
            7:"オープンされていません",
            8:"パラメータエラー",
            9:"USB通信エラーです"
}
def GetErrMessage(retcode):
    if( retcode in ErrStrDic):
        return ErrStrDic[retcode]
    else:
        return "不明なエラーです"